"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsumerEventListener = void 0;
const consumer_message_rate_1 = require("./consumer/consumer-message-rate");
const consumer_message_rate_writer_1 = require("./consumer/consumer-message-rate-writer");
const redis_smq_1 = require("redis-smq");
const redis_smq_common_1 = require("redis-smq-common");
class ConsumerEventListener {
    constructor() {
        this.consumerId = null;
        this.consumerMessageRate = new Map();
        this.redisClient = null;
        this.eventProvider = null;
        this.onMessageAcknowledged = (message) => {
            this.getMessageRate(message.getDestinationQueue(), String(this.consumerId)).incrementAcknowledged();
        };
        this.onMessageDeadLettered = (message) => {
            this.getMessageRate(message.getDestinationQueue(), String(this.consumerId)).incrementDeadLettered();
        };
    }
    getMessageRate(queue, consumerId) {
        let messageRate = this.consumerMessageRate.get(queue);
        if (!messageRate) {
            const redisClient = this.getRedisClient();
            this.redisClient = redisClient;
            const writer = new consumer_message_rate_writer_1.ConsumerMessageRateWriter(redisClient, queue, consumerId);
            messageRate = new consumer_message_rate_1.ConsumerMessageRate(writer);
            this.consumerMessageRate.set(queue, messageRate);
        }
        return messageRate;
    }
    getRedisClient() {
        if (!this.redisClient)
            throw new redis_smq_common_1.errors.PanicError('Expected a RedisClient instance');
        return this.redisClient;
    }
    init(args, cb) {
        const { instanceId, config, eventProvider } = args;
        this.consumerId = instanceId;
        this.eventProvider = eventProvider;
        (0, redis_smq_common_1.createClientInstance)(config.redis, (err, redisClient) => {
            if (err)
                cb(err);
            else if (!redisClient)
                cb(new redis_smq_common_1.errors.EmptyCallbackReplyError());
            else {
                this.redisClient = redisClient;
                eventProvider.on(redis_smq_1.events.MESSAGE_ACKNOWLEDGED, this.onMessageAcknowledged);
                eventProvider.on(redis_smq_1.events.MESSAGE_DEAD_LETTERED, this.onMessageDeadLettered);
                cb();
            }
        });
    }
    quit(cb) {
        var _a, _b;
        (_a = this.eventProvider) === null || _a === void 0 ? void 0 : _a.removeListener(redis_smq_1.events.MESSAGE_ACKNOWLEDGED, this.onMessageAcknowledged);
        (_b = this.eventProvider) === null || _b === void 0 ? void 0 : _b.removeListener(redis_smq_1.events.MESSAGE_DEAD_LETTERED, this.onMessageDeadLettered);
        redis_smq_common_1.async.waterfall([
            (cb) => {
                redis_smq_common_1.async.eachOf([...this.consumerMessageRate.values()], (messageRate, idx, done) => messageRate.quit(done), (err) => {
                    if (err)
                        cb(err);
                    else {
                        this.consumerMessageRate.clear();
                        cb();
                    }
                });
            },
            (cb) => {
                if (this.redisClient)
                    this.redisClient.halt(cb);
                else
                    cb();
            },
        ], cb);
    }
}
exports.ConsumerEventListener = ConsumerEventListener;
//# sourceMappingURL=consumer-event-listener.js.map