"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MonitorServer = void 0;
require("reflect-metadata");
const http_1 = require("http");
const Koa = require("koa");
const socket_io_1 = require("socket.io");
const KoaBodyParser = require("koa-bodyparser");
const redis_smq_monitor_client_1 = require("redis-smq-monitor-client");
const uuid_1 = require("uuid");
const cors = require("@koa/cors");
const stoppable = require("stoppable");
const bluebird_1 = require("bluebird");
const error_handler_1 = require("./middlewares/error-handler");
const routing_1 = require("./common/routing");
const api_controller_1 = require("./controllers/api/api-controller");
const redis_keys_1 = require("./common/redis-keys/redis-keys");
const redis_smq_1 = require("redis-smq");
const redis_smq_common_1 = require("redis-smq-common");
const redis_1 = require("./lib/redis");
const logger_1 = require("./lib/logger");
const registry_1 = require("./lib/registry");
const queue_manager_1 = require("./lib/queue-manager");
const message_manager_1 = require("./lib/message-manager");
const time_series_clean_up_worker_1 = require("./workers/time-series.clean-up.worker");
const websocket_heartbeat_stream_worker_1 = require("./workers/websocket-heartbeat-stream.worker");
const websocket_main_stream_worker_1 = require("./workers/websocket-main-stream.worker");
const websocket_online_stream_worker_1 = require("./workers/websocket-online-stream.worker");
const websocket_rate_stream_worker_1 = require("./workers/websocket-rate-stream.worker");
const fan_out_exchange_manager_1 = require("./lib/fan-out-exchange-manager");
class MonitorServer {
    constructor(config, logger) {
        this.workerRunner = null;
        this.application = null;
        this.redisClient = null;
        this.subscribeClient = null;
        this.instanceId = (0, uuid_1.v4)();
        this.config = config;
        this.logger = logger;
        this.powerManager = new redis_smq_common_1.PowerManager(false);
        registry_1.registry.register('config', this.config);
        registry_1.registry.register('logger', this.logger);
    }
    getRequiredProperty(key) {
        if (!this[key]) {
            throw new redis_smq_common_1.errors.PanicError(`Expected a non null value of key [${key}]`);
        }
        return this[key];
    }
    getApplication() {
        if (!this.application) {
            throw new redis_smq_common_1.errors.PanicError(`Expected a non null value.`);
        }
        return this.application;
    }
    getSubscribeClient() {
        if (!this.subscribeClient) {
            throw new redis_smq_common_1.errors.PanicError(`Expected a non null value.`);
        }
        return this.subscribeClient;
    }
    getRedisClient() {
        if (!this.redisClient) {
            throw new redis_smq_common_1.errors.PanicError(`Expected a non null value.`);
        }
        return this.redisClient;
    }
    getWorkerRunner() {
        if (!this.workerRunner) {
            throw new redis_smq_common_1.errors.PanicError(`Expected a non null value.`);
        }
        return this.workerRunner;
    }
    bootstrap() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            this.redisClient = yield (0, redis_1.createRedisInstance)(this.config);
            registry_1.registry.register('redis', this.redisClient);
            const { socketOpts = {}, basePath } = (_a = this.config.server) !== null && _a !== void 0 ? _a : {};
            const app = new Koa();
            app.use(error_handler_1.errorHandler);
            app.use(KoaBodyParser());
            app.use((0, redis_smq_monitor_client_1.Middleware)(['/api/', '/socket.io/'], basePath));
            app.use(cors({
                origin: '*',
            }));
            const router = (0, routing_1.getApplicationRouter)(app, [api_controller_1.apiController]);
            app.use(router.routes());
            app.use(router.allowedMethods());
            const httpServer = stoppable((0, http_1.createServer)(app.callback()));
            const socketIO = new socket_io_1.Server(httpServer, Object.assign(Object.assign({}, socketOpts), { cors: {
                    origin: '*',
                } }));
            this.application = {
                httpServer,
                socketIO,
                app,
            };
            return this.application;
        });
    }
    subscribe(socketIO) {
        return __awaiter(this, void 0, void 0, function* () {
            this.subscribeClient = yield (0, redis_1.createRedisInstance)(this.config);
            this.subscribeClient.psubscribe('stream*');
            this.subscribeClient.on('pmessage', (pattern, channel, message) => {
                socketIO.emit(channel, JSON.parse(message));
            });
        });
    }
    runWorkers() {
        return __awaiter(this, void 0, void 0, function* () {
            const { keyLockMonitorWorkers } = redis_keys_1.redisKeys.getMainKeys();
            this.workerRunner = new redis_smq_common_1.WorkerRunner(this.getRedisClient(), keyLockMonitorWorkers, new redis_smq_common_1.WorkerPool(), this.logger);
            this.workerRunner.on(redis_smq_1.events.ERROR, (err) => {
                throw err;
            });
            const redisClient = this.getRedisClient();
            const queueManager = yield (0, queue_manager_1.getQueueManagerInstance)(this.config);
            const messageManager = yield (0, message_manager_1.getMessageManagerInstance)(this.config);
            this.workerRunner.addWorker(new time_series_clean_up_worker_1.default(redisClient, queueManager, true));
            this.workerRunner.addWorker(new websocket_heartbeat_stream_worker_1.default(redisClient, true));
            this.workerRunner.addWorker(new websocket_main_stream_worker_1.default(redisClient, messageManager, queueManager, true));
            this.workerRunner.addWorker(new websocket_online_stream_worker_1.default(redisClient, queueManager, true));
            this.workerRunner.addWorker(new websocket_rate_stream_worker_1.default(redisClient, queueManager, true));
            yield new Promise((resolve) => {
                var _a, _b;
                (_a = this.workerRunner) === null || _a === void 0 ? void 0 : _a.once(redis_smq_1.events.UP, resolve);
                (_b = this.workerRunner) === null || _b === void 0 ? void 0 : _b.run();
            });
        });
    }
    listen() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const { host = '0.0.0.0', port = 7210 } = (_a = this.config.server) !== null && _a !== void 0 ? _a : {};
            const r = this.powerManager.goingUp();
            if (r) {
                this.logger.info('Going up...');
                const { socketIO, httpServer } = yield this.bootstrap();
                yield this.subscribe(socketIO);
                yield this.runWorkers();
                yield new Promise((resolve) => {
                    httpServer.listen(port, host, resolve);
                });
                this.powerManager.commit();
                this.logger.info(`Instance ID is ${this.instanceId}.`);
                this.logger.info(`Up and running on ${host}:${port}...`);
                return true;
            }
            return false;
        });
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            const r = this.powerManager.goingDown();
            if (r) {
                this.logger.info('Going down...');
                const { httpServer } = this.getApplication();
                yield new Promise((resolve) => httpServer.stop(resolve));
                yield (0, bluebird_1.promisifyAll)(this.getWorkerRunner()).quitAsync();
                yield (0, bluebird_1.promisifyAll)(this.getSubscribeClient()).haltAsync();
                yield (0, bluebird_1.promisifyAll)(this.getRedisClient()).haltAsync();
                yield (0, message_manager_1.destroyMessageManager)();
                yield (0, queue_manager_1.destroyQueueManager)();
                yield (0, fan_out_exchange_manager_1.destroyFanOutExchangeManager)();
                registry_1.registry.reset();
                this.workerRunner = null;
                this.application = null;
                this.powerManager.commit();
                this.logger.info('Down.');
                return true;
            }
            return false;
        });
    }
    static createInstance(config = {}) {
        if (!MonitorServer.instance) {
            const logger = (0, logger_1.getNamespacedLoggerInstance)(config, 'monitor');
            MonitorServer.instance = new MonitorServer(config, logger);
        }
        return MonitorServer.instance;
    }
}
exports.MonitorServer = MonitorServer;
MonitorServer.instance = null;
//# sourceMappingURL=index.js.map