"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebsocketMainStreamWorker = void 0;
const redis_smq_common_1 = require("redis-smq-common");
const redis_smq_1 = require("redis-smq");
class WebsocketMainStreamWorker extends redis_smq_common_1.Worker {
    constructor(redisClient, messageManager, queueManager, managed) {
        super(managed);
        this.data = {
            scheduledMessagesCount: 0,
            deadLetteredMessagesCount: 0,
            pendingMessagesCount: 0,
            acknowledgedMessagesCount: 0,
            consumersCount: 0,
            queuesCount: 0,
            queues: {},
        };
        this.addQueue = (queue, settings) => {
            const { ns, name } = queue;
            const { priorityQueuing, rateLimit } = settings;
            if (!this.data.queues[ns]) {
                this.data.queues[ns] = {};
            }
            if (!this.data.queues[ns][name]) {
                this.data.queuesCount += 1;
                this.data.queues[ns][name] = {
                    name,
                    ns,
                    priorityQueuing,
                    rateLimit: rateLimit !== null && rateLimit !== void 0 ? rateLimit : null,
                    deadLetteredMessagesCount: 0,
                    acknowledgedMessagesCount: 0,
                    pendingMessagesCount: 0,
                    consumersCount: 0,
                };
            }
            return this.data.queues[ns][name];
        };
        this.getQueueSize = (queues, cb) => {
            redis_smq_common_1.async.each(queues, (queueParams, _, done) => {
                redis_smq_common_1.async.waterfall([
                    (cb) => {
                        this.queueManager.queue.getSettings(queueParams, (err, settings) => {
                            if (err)
                                done(err);
                            else if (!settings)
                                cb(new redis_smq_common_1.errors.EmptyCallbackReplyError());
                            else {
                                const queue = this.addQueue(queueParams, settings);
                                cb(null, queue);
                            }
                        });
                    },
                    (queue, cb) => {
                        this.messageManager.pendingMessages.count(queueParams, (err, count) => {
                            if (err)
                                cb(err);
                            else {
                                queue.pendingMessagesCount = Number(count);
                                this.data.pendingMessagesCount +=
                                    queue.pendingMessagesCount;
                                cb(null, queue);
                            }
                        });
                    },
                    (queue, cb) => {
                        this.messageManager.deadLetteredMessages.count(queueParams, (err, count) => {
                            if (err)
                                cb(err);
                            else {
                                queue.deadLetteredMessagesCount = Number(count);
                                this.data.deadLetteredMessagesCount +=
                                    queue.deadLetteredMessagesCount;
                                cb(null, queue);
                            }
                        });
                    },
                    (queue, cb) => {
                        this.messageManager.acknowledgedMessages.count(queueParams, (err, count) => {
                            if (err)
                                cb(err);
                            else {
                                queue.acknowledgedMessagesCount = Number(count);
                                this.data.acknowledgedMessagesCount +=
                                    queue.acknowledgedMessagesCount;
                                cb();
                            }
                        });
                    },
                ], done);
            }, cb);
        };
        this.getQueues = (cb) => {
            this.queueManager.queue.list(cb);
        };
        this.countScheduledMessages = (cb) => {
            this.messageManager.scheduledMessages.count((err, count) => {
                if (err)
                    cb(err);
                else {
                    this.data.scheduledMessagesCount = count !== null && count !== void 0 ? count : 0;
                    cb();
                }
            });
        };
        this.countQueueConsumers = (queue, cb) => {
            redis_smq_1.Consumer.countOnlineConsumers(this.redisClient, queue, (err, reply) => {
                if (err)
                    cb(err);
                else {
                    const { ns, name } = queue;
                    const count = Number(reply);
                    this.data.consumersCount += count;
                    this.data.queues[ns][name].consumersCount = count;
                    cb();
                }
            });
        };
        this.updateOnlineInstances = (cb) => {
            redis_smq_common_1.async.each(this.data.queues, (item, key, done) => {
                redis_smq_common_1.async.each(item, (item, key, done) => {
                    this.countQueueConsumers(item, done);
                }, done);
            }, cb);
        };
        this.publish = (cb) => {
            this.redisClient.publish('streamMain', JSON.stringify(this.data), () => cb());
        };
        this.reset = () => {
            this.data = {
                scheduledMessagesCount: 0,
                deadLetteredMessagesCount: 0,
                pendingMessagesCount: 0,
                acknowledgedMessagesCount: 0,
                consumersCount: 0,
                queuesCount: 0,
                queues: {},
            };
        };
        this.work = (cb) => {
            this.reset();
            redis_smq_common_1.async.waterfall([
                this.countScheduledMessages,
                this.getQueues,
                this.getQueueSize,
                this.updateOnlineInstances,
            ], (err) => {
                if (err)
                    cb(err);
                else {
                    this.publish(cb);
                }
            });
        };
        this.queueManager = queueManager;
        this.messageManager = messageManager;
        this.redisClient = redisClient;
    }
}
exports.WebsocketMainStreamWorker = WebsocketMainStreamWorker;
exports.default = WebsocketMainStreamWorker;
//# sourceMappingURL=websocket-main-stream.worker.js.map